import unittest
import datetime
import time
from unix_dates import UnixDate, UnixTimeDelta


class TestUnixDate(unittest.TestCase):
    def test_naive(self):
        now = datetime.datetime.now()

        unix_time = UnixDate.to_unix_time(now)

        back_datetime = UnixDate.to_datetime(unix_time, tz=None)
        self.assertEqual(back_datetime.year, now.year)
        self.assertEqual(back_datetime.month, now.month)
        self.assertEqual(back_datetime.day, now.day)
        self.assertEqual(back_datetime.hour, now.hour)
        self.assertEqual(back_datetime.minute, now.minute)
        self.assertEqual(back_datetime.second, now.second)

        back_unix_time = UnixDate.to_unix_time(back_datetime)
        self.assertEqual(back_unix_time, unix_time)
        self.assertEqual(UnixDate.to_datetime(back_unix_time, tz=None), back_datetime)

    def test_not_naive(self):
        now = datetime.datetime.now(tz=UnixDate.LOCAL_UTC)

        unix_time = UnixDate.to_unix_time(now)

        back_datetime = UnixDate.to_datetime(unix_time)
        self.assertEqual(back_datetime.year, now.year)
        self.assertEqual(back_datetime.month, now.month)
        self.assertEqual(back_datetime.day, now.day)
        self.assertEqual(back_datetime.hour, now.hour)
        self.assertEqual(back_datetime.minute, now.minute)
        self.assertEqual(back_datetime.second, now.second)

        back_unix_time = UnixDate.to_unix_time(back_datetime)
        self.assertEqual(back_unix_time, unix_time)
        self.assertEqual(UnixDate.to_datetime(back_unix_time), back_datetime)

    def test_time_zeons(self):
        utc = datetime.datetime(2016, 1, 1, 12, 0, 0, 0, tzinfo=UnixDate.UTC)
        local = datetime.datetime(2016, 1, 1, 12, 0, 0, 0, tzinfo=UnixDate.LOCAL_UTC)

        utc_offset_seconds = (time.timezone if (time.localtime().tm_isdst == 0) else time.altzone)
        utc_offset_hours = utc_offset_seconds / (60 * 60)

        self.assertEqual((local - utc).seconds, utc_offset_seconds)

        utc = datetime.datetime(2016, 1, 1, 12 + utc_offset_hours, 0, 0, 0, tzinfo=UnixDate.UTC)
        local = datetime.datetime(2016, 1, 1, 12, 0, 0, 0, tzinfo=UnixDate.LOCAL_UTC)

        self.assertEqual((local - utc).seconds, 0)
        utc_ut = UnixDate.to_unix_time(utc)
        local_ut = UnixDate.to_unix_time(local)
        self.assertEqual(local_ut, utc_ut)

    def test_parsing(self):
        d = UnixDate.to_unix_time_from_iso_format("2008-09-03T20:56:35.450686Z")
        dt = UnixDate.to_naive_datetime(d)

        self.assertEqual(dt.year, 2008)
        self.assertEqual(dt.month, 9)
        self.assertEqual(dt.day, 3)
        self.assertEqual(dt.hour, 20)
        self.assertEqual(dt.minute, 56)
        self.assertEqual(dt.second, 35)

        d = UnixDate.to_unix_time_from_aws_format("2015-12-24T16:46:19.166+0000")
        dt = UnixDate.to_naive_datetime(d)

        self.assertEqual(dt.year, 2015)
        self.assertEqual(dt.month, 12)
        self.assertEqual(dt.day, 24)
        self.assertEqual(dt.hour, 16)
        self.assertEqual(dt.minute, 46)
        self.assertEqual(dt.second, 19)

    def test_delta(self):
        self.assertEqual(UnixTimeDelta.calc(millis=1000), 1)
        self.assertEqual(UnixTimeDelta.calc(seconds=1), 1)
        self.assertEqual(UnixTimeDelta.calc(minutes=1), 60)
        self.assertEqual(UnixTimeDelta.calc(hours=1), 3600)
        self.assertEqual(UnixTimeDelta.calc(days=1), 3600 * 24)

    def test_time_zones(self):
        utc = datetime.datetime(2016, 1, 1, 12, 0, 0, 0, tzinfo=UnixDate.UTC)
        local = datetime.datetime(2016, 1, 1, 12, 0, 0, 0, tzinfo=UnixDate.LOCAL_UTC)

        utc_offset_seconds = (time.timezone if (time.localtime().tm_isdst == 0) else time.altzone)
        utc_offset_hours = utc_offset_seconds / (60 * 60)

        self.assertEqual((local - utc).seconds, utc_offset_seconds)

        utc = datetime.datetime(2016, 1, 1, 12 + utc_offset_hours, 0, 0, 0, tzinfo=UnixDate.UTC)
        local = datetime.datetime(2016, 1, 1, 12, 0, 0, 0, tzinfo=UnixDate.LOCAL_UTC)

        self.assertEqual((local - utc).seconds, 0)
        utc_ut = UnixDate.to_unix_time(utc)
        local_ut = UnixDate.to_unix_time(local)
        self.assertEqual(local_ut, utc_ut)

    def test_hour_of_the_day(self):
        utc = datetime.datetime(2016, 1, 1, 14, 0, 0, 0, tzinfo=UnixDate.UTC)
        utc_ut = UnixDate.to_unix_time(utc)
        hour_of_the_day = UnixTimeDelta.hour_of_the_day(utc_ut)
        self.assertEqual(hour_of_the_day, utc.hour)

    def test_to_round_hour(self):
        utc_with_sec = datetime.datetime(2016, 1, 1, 14, 3, 3, 1)
        utc_ut_with_sec = UnixDate.to_unix_time(utc_with_sec)
        utc_without_sec = datetime.datetime(2016, 1, 1, 14, 0, 0, 0)
        utc_ut_without_sec = UnixDate.to_unix_time(utc_without_sec)

        round_hour = UnixTimeDelta.to_round_hour(utc_ut_with_sec)
        self.assertEqual(round_hour, utc_ut_without_sec)



import datetime
import time


__version__ = 0.4


class UnixDate(object):
    ISO_DATE_FMT = "%Y-%m-%dT%H:%M:%S"
    AWS_DATE_FORMAT = "%Y-%m-%dT%H:%M:%S.%f+0000"
    UTC_OFFSET_SECONDS = -1 * (time.timezone if (time.localtime().tm_isdst == 0) else time.altzone)

    """
    Utilities to convert dates to Unix time and back.

    Unix timestamps are the format we use to store dates in the system. This is a consistent way to store dates - cross
    time-zones, efficiently, without having to parse strings back and forth.
    """

    class UTCLocalTimeZoneInfo(datetime.tzinfo):
        def utcoffset(self, dt):
            return datetime.timedelta(seconds=UnixDate.UTC_OFFSET_SECONDS)

        def dst(self, dt):
            return datetime.timedelta(0)

    class UTCTimeZoneInfo(datetime.tzinfo):
        def utcoffset(self, dt):
            return datetime.timedelta(seconds=0)

        def dst(self, dt):
            return datetime.timedelta(0)

    LOCAL_UTC = UTCLocalTimeZoneInfo()
    UTC = UTCTimeZoneInfo()

    @classmethod
    def now(cls):
        """
        :rtype: float
        :return: The Unix epoch: number of seconds that have elapsed since January 1, 1970 (UTC)
        """
        return cls.to_unix_time(datetime.datetime.now())

    @classmethod
    def to_unix_time(cls, dt):
        """
        convert datetime to epoch
        :type dt: datetime.datetime
        :rtype: float
        :return: The Unix epoch: number of seconds that have elapsed since January 1, 1970
        """

        timetuple = dt.timetuple()
        unix_time = time.mktime(timetuple)

        if dt.tzinfo:
            # if it is not a naive, add the time zone
            utcoffset = dt.tzinfo.utcoffset(dt)
            seconds = utcoffset.days * 24 * 60 * 60 + utcoffset.seconds
            unix_time -= seconds - cls.UTC_OFFSET_SECONDS

        return unix_time

    @classmethod
    def to_unix_time_from_iso_format(cls, date_str):
        """
        Parse ISO formatted strings back into unix timestamp

        :param date_str: String of date in ISO format
        :type date_str: str|unicode
        :rtype: float
        :return: The Unix epoch: number of seconds that have elapsed since January 1, 1970
        """

        if isinstance(date_str, (unicode, str,)):
            # is on 0 milliseconds
            # here are few solutions:
            #   http://stackoverflow.com/questions/30584364/python-strptime-format-with-optional-bits

            no_ms = date_str.split('.')[0]
            value = datetime.datetime.strptime(no_ms, cls.ISO_DATE_FMT)
            return cls.to_unix_time(value)

        else:
            raise ValueError("Failed convert {} {} to unix time".format(str.__class__, date_str))

    @classmethod
    def to_unix_time_from_aws_format(cls, date_str):
        """
        Parse AWS date formats back into unix timestamp

        :param date_str: String of date in ISO format
        :type date_str: str
        :rtype: float
        :return: The Unix epoch: number of seconds that have elapsed since January 1, 1970
        """
        return cls.to_unix_time(datetime.datetime.strptime(date_str, cls.AWS_DATE_FORMAT))

    @classmethod
    def to_datetime(cls, unix_time_sec, tz=LOCAL_UTC):
        """
        :type unix_time_sec: float
        :type tz: datetime.tzinfo
        :param tz: When None, will create an 'offset naive date'. Otherwise, return 'offset aware date'. Default to
                   convert to the platform locale
        :rtype: datetime.datetime
        """
        return datetime.datetime.fromtimestamp(timestamp=unix_time_sec, tz=tz)

    @classmethod
    def to_naive_datetime(cls, unix_time_sec):
        """
        :type unix_time_sec: float
        :rtype: datetime.datetime
        """

        return datetime.datetime.fromtimestamp(timestamp=unix_time_sec)


class UnixTimeDelta(object):
    """
    Convert time delta to unix time (allow easy calculations with UnixDate)
    """
    SECONDS_IN_HOUR = 60 * 60
    SECONDS_IN_DAY = 24 * SECONDS_IN_HOUR

    @classmethod
    def calc(cls, days=0, hours=0, minutes=0, seconds=0, millis=0):
        return days * cls.SECONDS_IN_DAY + \
               hours * cls.SECONDS_IN_HOUR + \
               minutes * 60 + \
               seconds + \
               millis / 1000

    @classmethod
    def hour_of_the_day(cls, unix_time_sec):
        """
        return the hour of the day based 0
        :type unix_time_sec: float
        :rtype: int
        """

        return int((unix_time_sec % (24 * 60 * 60)) / (60 * 60))


    @classmethod
    def to_round_hour(cls, unix_time_sec):
        """
        return the hour of the day based 0
        :type unix_time_sec: float
        :rtype: int
        """

        unix_time_sec -= unix_time_sec % 3600  # make it round hour
        return unix_time_sec





from setuptools import setup
from os import path
from unix_dates import __version__

here = path.abspath(path.dirname(__file__))

setup(
    name='unix_dates',
    version=__version__,
    description='Python Unix Dates conversion utilities',
    url='https://github.com/ophirh/python-unix-dates',
    author='Ophir',
    author_email='opensource@itculate.io',
    license='MIT',
    keywords=['unix', 'dates', 'utc', 'timestamp'],
    packages=['unix_dates'],
)
